// ----------------------------------------------------------------------------
/**
 
   Property of Kartech  
   (c) Copyright Kartech 2011

   Regular user license : 

       Regular users of Step2Mach are expected to register the program by paying for it ( Paypal, 35$ + VAT in European Union countries). 

       https://www.paypal.com/cgi-bin/webscr?cmd=_cart&add=1&business=jaroslaw%2Ekarwik%40kartech%2Eorg&item_name=Step2Mach%20software%20license&amount=35&currency_code=USD&lc=US&charset=windows-1250&button_subtype=products 
   
       Source code and bug report/upgrade info will be available on Mach Support forum.
       If you're a registered user, you can do anything you want with the program and source code to modify it as you see fit, except for redistribution. 
       In your own shop, anything goes.

   Professional license: 
   
       If you want to develop your commercial product based on the software, you need to buy special license ( Paypal, 1000$ + VAT in European Union countries) . 
       In such case you get rights to redistribute your product. In binary form, without disclosing your source code.
       You will also get access to premium features like ethernet converter source code and bootloader source code.
       For such case contact me first please - I will generate Paypal invoice for you.
  

   Author:      Jarosaw Karwik
   E-Mail:     jaroslaw.karwik(at)kartech.org
   
**/
// ----------------------------------------------------------------------------

#pragma once

#include "stdafx.h"
#include "types.h"

#include "mach_firmware.h"
#include "mach_pclib_protocol.h"



typedef struct
{
	double		T;
    double		S;
    Uint32_t    Cnt;
}LineVectorType;

typedef struct
{
    Uint32_t    Cnt;
    Int32_t     Dir;

    double V;
    double Vstart;
    double Vstop;
    double Acc;
    double Dcc;

	LineVectorType LineAcc;
	LineVectorType LineConst;
	LineVectorType LineDcc;

}VectorSpeedType;

class MyHardwareClass
{
private:
	int       steps_fw[4];
	double    steps_accumulated[4];
	char      dev_address[128];
	double	  move_tick;
	Uint32_t  move_tick_ms;
	int		  connected;	

	Uint32_t  CalcSpeed(int delta);


	pin_hw_e PinNr2Fn(int pin_nr,int polarity);

	void	CalculateAxis(VectorSpeedType * VS);
	void	PrepareAxisData(int	axis,Uint32_t Frequency,VectorSpeedType *  Vs,timer_step_t	*  AccVector,timer_step_t *  ConstVector,timer_step_t *  DccVector);

public:

	MyHardwareClass(double move_tick,const char * address);	
	virtual ~MyHardwareClass();


	// GCode moves processing
	Uint32_t	GetLineId(void);
	int		    AddMove(double ex,double ey,double ez,double ea,Uint32_t line_id);

	// IO control
	unsigned int GetInputs(void);
	void         SetOutputs(unsigned int outputs,unsigned int mask);
	void         UpdateCoords(int Xc,int Yc,int Zc, int Ac);

	// Jog
	void		StartJogMove(int axis,int distance,double  JogVel, double JogAcc,int PinStopMask,int PinStopValue);
	void		FinishJogMove(void);

	// Proces control
	int			ReadDeviceFeedback(int * coords);
	bool		IsIdle();
	int			ConfigureStepPins(int axis,int step_pin_nr,int step_polarity,int dir_pin_nr,int dir_polarity);
	int			Reset(void);
	void        Stop(void);

	// Debugging info
	int			GetStats(char * buffer,int buffer_size,int line);

};