// ----------------------------------------------------------------------------
/**
 
   Property of Kartech  
   (c) Copyright Kartech 2011

   Regular user license : 

       Regular users of Step2Mach are expected to register the program by paying for it ( Paypal, 35$ + VAT in European Union countries). 

       https://www.paypal.com/cgi-bin/webscr?cmd=_cart&add=1&business=jaroslaw%2Ekarwik%40kartech%2Eorg&item_name=Step2Mach%20software%20license&amount=35&currency_code=USD&lc=US&charset=windows-1250&button_subtype=products 
   
       Source code and bug report/upgrade info will be available on Mach Support forum.
       If you're a registered user, you can do anything you want with the program and source code to modify it as you see fit, except for redistribution. 
       In your own shop, anything goes.

   Professional license: 
   
       If you want to develop your commercial product based on the software, you need to buy special license ( Paypal, 1000$ + VAT in European Union countries) . 
       In such case you get rights to redistribute your product. In binary form, without disclosing your source code.
       You will also get access to premium features like ethernet converter source code and bootloader source code.
       For such case contact me first please - I will generate Paypal invoice for you.
  

   Author:      Jarosaw Karwik
   E-Mail:     jaroslaw.karwik(at)kartech.org
   
**/
// ----------------------------------------------------------------------------

#pragma once

#include "stdafx.h"
#include "MachDevice.h"
#include "myConsole.h"
#include "myHardware.h"

#define  MOTION_TIME_SLICE		0.004 /* in seconds */


typedef enum
{
	SEQ_IDLE,
	SEQ_WAIT_FOR_IDLE,
	SEQ_DWELLING,
	SEQ_HOME_FIND,
	SEQ_HOME_DEREF,
	SEQ_HOME_FINISHED

}MySequenceEnum;

typedef struct
{
	unsigned int	JogOnCall;
	unsigned int	JogOffCall;
	unsigned int	DwellCall;
	unsigned int	HomeCall;
	unsigned int	ProbeCall;
	unsigned int	PurgeCall;
	unsigned int	ResetCall;
	unsigned long	TrajectoryItems;

}MyDeviceStatsType;



class MyDeviceClass
{
private:
	
	int					  ReadyForRun;
	MyConsoleClass		* console;
	MyHardwareClass		* hardware;
	std::string			  dev_ip_addr;
	MyDeviceStatsType	  stats;

	// Main state machine variables
	MySequenceEnum		  sequence;
	int				      InMotion;
	int					  WaitForGcodeIdle;
	int					  WaitForJogIdle;
	
	// For servicing Dwell requests
	DWORD				  DwellTimestamp;
	DWORD				  DwellDelay;

	// For servicing Jog requests
	int				      JogAxis;
    double				  JogFeed;
    double				  JogVel;
    double				  JogAcc;
	int					  JogDistance;

	// For servicing Home requests
	int				      HomeAxis;
	int					  HomePin;
	int					  HomeSignal;
	double				  HomeVel;
	double				  HomeAcc;
	int					  HomeMask;
	int					  HomeMaskValActive;
	int					  HomeMaskValInActive;
	int					  HomeDistance;
	int					  HomeDelay;

	// For servicing outputs update
	int					  OutputValid;
	int					  OutputCurrent;
	int					  OutputMaskCurrent;

	// Licensing
	int					  LicenseValid;


	void  SendHoldingMovement(void);
	void  GetInputs(void);
	void  SetOutputs(void);
	void  HandleSequences(void);
	void  ConfigureHardware(void);

	void  DumpConfig(void);

public:
	MyDeviceClass();
	virtual ~MyDeviceClass();


	void  InitControl(void);
	void  SetLicense(int valid);
	void  Reconfigure(std::string ip_addr,bool debug_mode);
	void  SetMachError(CString  Message);
	void  JogOn( short axis, short direction, double SpeedNorm);
	void  JogOff(short axis);
	void  Dwell(double time);
	void  Home(short axis);
	void  Probe(void);
	void  Purge(short flags);
	void  Reset(void);
    void  Update(void);	

};
